// This is a ZeroMQ client. Messages are routed to zeromq workers living inside the JVM.

var zeromq = require('zeromq');
var common = require("./common");
var params = common.params;


// we bind a zmq socket. The JVM workers will connect to it.
var socket = zeromq.createSocket("xreq");
var portBase = 6400;
var portEnd = portBase + 20;

function bind(port){
    socket.bind("tcp://0.0.0.0:" + port, function(err) {
        if(err) {
            if(port < portEnd)
                bind(port + 1);
            else
                console.log("fatal: cannot bind to ports from " + portBase + " to " + portEnd + " due to " + e);
        } else {
            console.log("bound to port " + port);
        }
    });
}

bind(portBase);

// We need to keep track of outstanding requests. Each outgoing request
// will have an id and we'll register callbacks here.
var LRU = require("lru-cache");
var callbacks = LRU(1000);

var requestId = 0;

// When the worker responds to a request we get this event. We delegate the response
// to the callback registered when sending the job.
socket.on("message", function(rid, blank, data) { 
    var cb = callbacks.get(rid);
    if(cb)
        cb(data); 
    else
        //console.log("unknown reply " + id + ", data: " + data);
        console.log("unknown reply " + rid + ", max: " + callbacks.maxLength + ", length: " + callbacks.length);
});

var pid = "" + process.pid + "-";

// We send an message into the zmq queue, and render whatever comes back.
// The parameters are encoded as JSON.
function zmqSend(res, params) {
    var rid = pid+requestId++;

    callbacks.set(rid, function(data) {
        common.reply(res)(null, data);
        callbacks.del(rid);
    });

    socket.send(rid, "", JSON.stringify(params));
}

// Constructor four our bridge object.
function ZmqClient(print_stats) {
    this.print_stats = print_stats;
}

ZmqClient.prototype = {
    init: function() {
        if(this.print_stats)
            setInterval(function() { console.log("request queue length: " + callbacks.length); }, 5000);
    },
    
    indexLookup: function(req, res, next) {
        zmqSend(res, {method: "indexLookup", params: params(req)});
    },

    getBrowsingStatistics: function(req, res, next) {
        zmqSend(res, {method: "getBrowsingStatistics", params: params(req)});
    },

    getNumberOfElements: function(req, res, next) {
        zmqSend(res, {method: "getNumberOfElements", params: params(req)});
    },

    getResult:  function(req, res, next) {
        zmqSend(res, {method: "getResult", params: params(req)});
    },

    stats: function(req, res, next) {
        res.end("callback queue length: " + callbacks.length);
    }
};

exports.Client = ZmqClient;
